import json
import logging
import urllib.parse

import openai
import requests
from wrapt import wrap_function_wrapper

from .base_instrumentor import BaseInstrumentor

logger = logging.getLogger("qualifire")


class OpenAiInstrumentor(BaseInstrumentor):
    WRAPPED_METHODS = [
        {
            "func": openai.ChatCompletion.create,
            "object": "ChatCompletion",
            "method": "create",
            "span_name": "openai.chat",
        },
        {
            "func": openai.Completion.create,
            "object": "Completion",
            "method": "create",
            "span_name": "openai.completion",
        },
    ]

    def __init__(
        self,
        base_url: str,
        api_key: str,
    ):
        self._base_url = base_url
        self._api_key = api_key

    def _wrap(self, func, instance, args, kwargs):
        if hasattr(func, "__wrapped__"):
            return func(*args, **kwargs)

        headers = {
            "Content-type": "application/json",
            "Accept": "application/json",
            "X-qualifire-key": self._api_key,
        }

        q_response = requests.post(
            urllib.parse.urljoin(self._base_url, "/api/intake"),
            data=json.dumps(
                {
                    "caller": f"{instance.__name__}.{func.__name__}",
                    "body": kwargs,
                }
            ),
            headers=headers,
            timeout=300,
        )

        response = func(*args, **kwargs)

        if q_response.json()["success"]:
            requests.patch(
                urllib.parse.urljoin(self._base_url, "/api/intake"),
                data=json.dumps(
                    {
                        "createdCallId": q_response.json()["id"],
                        "model": response.get("model"),
                        "body": response,
                    },
                ),
                headers=headers,
                timeout=300,
            )
        return response

    def initialize(
        self,
    ):
        for wrapped_method in self.WRAPPED_METHODS:
            wrap_object = wrapped_method.get("object")
            wrap_method = wrapped_method.get("method")
            logger.debug(
                "patching function",
                {
                    "wrap_method": wrap_method,
                },
            )
            wrap_function_wrapper(
                "openai",
                f"{wrap_object}.{wrap_method}",
                self._wrap,
            )



import logging

from .instrumentations import instrumentors

logger = logging.getLogger("qualifire")


class Client:
    def __init__(
        self,
        base_url,
        api_key,
    ) -> None:
        self._base_url = base_url
        self._api_key = api_key

    def initialize(self):
        for instrumentor_class in instrumentors:
            logger.debug(
                "initializing instrumentor",
                {
                    "instrumentor": instrumentor_class.__name__,
                },
            )
            try:
                instrumentor = instrumentor_class(
                    api_key=self._api_key,
                    base_url=self._base_url,
                )

                instrumentor.initialize()
            except Exception:
                logger.exception("error while initializing instrumentor.")

        logger.debug("initialized all instrumentors")



# type: ignore[attr-defined]
"""Qualifire PYthon SDK"""

import logging
import os
import sys

from . import client, instrumentations

logger = logging.getLogger("qualifire")
QUALIFIRE_API_KEY_ENV_VAR = "QUALIFIRE_API_KEY"
QUALIFIRE_BASE_URL_ENV_VAR = "QUALIFIRE_BASE_URL"
_DEFAULT_BASE_URL = "https://intake.qualifire.ai/"


from importlib import metadata as importlib_metadata


def get_version() -> str:
    try:
        return importlib_metadata.version(__name__)
    except importlib_metadata.PackageNotFoundError:  # pragma: no cover
        return "unknown"


version: str = get_version()


def init(
    api_key=None,
    debug=False,
    base_url=None,
):
    if api_key is None:
        api_key = os.environ.get(QUALIFIRE_API_KEY_ENV_VAR)
        if api_key is None:
            logger.warning(
                f"No API key found, please pass the api key to this function or set the environment variable: {QUALIFIRE_API_KEY_ENV_VAR}"
            )

    if debug is True:
        logger.setLevel("DEBUG")
    else:
        logger.setLevel("ERROR")

    if base_url is None:
        base_url = os.environ.get(QUALIFIRE_BASE_URL_ENV_VAR)

        if base_url is None:
            base_url = _DEFAULT_BASE_URL

    logger.debug(
        "initializing client",
        {
            "base_url": base_url,
        },
    )

    qualifire_client = client.Client(
        base_url=base_url,
        api_key=api_key,
    )

    try:
        qualifire_client.initialize()
    except Exception:
        logger.exception("Error while initializing")
